import { ELEMENTS_CLASS } from "../../../core/enums/elements-class-enum";
import { GeometricObject } from "../../../core/models/objects/geometric-object";
import { IntersectionModel } from "../../intersection-component/models/intersection-model";
export class CircumferenceModel extends GeometricObject {
    constructor(center, radius) {
        super();
        this.center = center;
        this.radius = radius;
        this._coordinates = [];
        this._coordinates[0] = this.center.posX;
        this._coordinates[1] = this.center.posY;
        this._coordinates[2] = this.radius.posX;
        this._coordinates[3] = this.radius.posY;
        super.setClass(ELEMENTS_CLASS.CIRCUMFERENCE);

    }
    getRadius() {
        this._coordinates[0] = this.center.posX;
        this._coordinates[1] = this.center.posY;
        this._coordinates[2] = this.radius.posX;
        this._coordinates[3] = this.radius.posY;
        const legA = this._coordinates[2] - this._coordinates[0];
        const legB = this._coordinates[3] - this._coordinates[1];
        const radius = Math.sqrt(Math.pow(legA, 2) + Math.pow(legB, 2));
        return radius;
    }

    getStraight() {
        const dx = this.radius.posX - this.center.posX;
        const dy = this.radius.posY - this.center.posY;
        const c = Math.sqrt((dy * dy) + (dx * dx));
        return [dx, dy, c];
    }

    getDirection() {
        const aX = this.center.posX;
        const aY = this.center.posY;
        const bX = this.radius.posX;
        const bY = this.radius.posY;
        const a = bX - aX;
        const b = bY - aY;
        return [a, b];
    }
    getIntersection(geometricObject) {
        switch (geometricObject.elementClass) {
            case ELEMENTS_CLASS.LINE_SEGMENT:
                return this.getIntersectionByLine(geometricObject);
            case ELEMENTS_CLASS.CIRCUMFERENCE:
                return this.getIntersectionByCircumference(geometricObject);
            default:
                break;
        }
    }
    getIntersectionByCircumference(circumference) {

        const c0 = this.getStraight();

        if (c0[2] > (this.radius + circumference.radius)) {
            return [new IntersectionModel(
                Number.MAX_SAFE_INTEGER,
                Number.MAX_SAFE_INTEGER,
                undefined,
                this,
                circumference,
                false
            ), new IntersectionModel(
                Number.MAX_SAFE_INTEGER,
                Number.MAX_SAFE_INTEGER,
                undefined,
                this,
                circumference,
                false
            )];
        }
        if (c0[2] < this.radius + circumference.radius) {
            return [new IntersectionModel(
                Number.MAX_SAFE_INTEGER,
                Number.MAX_SAFE_INTEGER,
                undefined,
                this,
                circumference,
                false
            ), new IntersectionModel(
                Number.MAX_SAFE_INTEGER,
                Number.MAX_SAFE_INTEGER,
                undefined,
                this,
                circumference,
                false
            )];
        }

        const distance = ((this.radius * this.radius) - (circumference.radius * circumference.radius) + (c0[2] * c0[2])) / (2.0 * c0[2]);
        const aX = this.center.posX + (c0[0] * distance / c0[2]);
        const aY = this.center.posY + (c0[1] * distance / c0[2]);
        const distance2 = Math.sqrt((this.radius * this.radius) - (distance * distance));
        const bX = -c0[0] * (distance2 / c0[2]);
        const bY = c0[1] * (distance2 / c0[2]);
        const intersectionAX = aX + bX;
        const intersectionBX = aX - bX;
        const intersectionAY = aY + bY;
        const intersectionBY = aY - bY;
        return [new IntersectionModel(
            intersectionAX,
            intersectionAY,
            undefined,
            this,
            circumference,
            false
        ), new IntersectionModel(
            intersectionBX,
            intersectionBY,
            undefined,
            this,
            circumference,
            false
        )];
    }


    getIntersectionByLine(lineSegment) {
        const r = this.getStraight();
        const s = lineSegment.getStraight();
        const a1 = r[0];
        const b1 = r[1];
        const c1 = r[2];
        const a2 = s[0];
        const b2 = s[1];
        const c2 = s[2];
        const determinant = a1 * b2 - a2 * b1;
        if (determinant == 0) {
            return new IntersectionModel(
                Number.MAX_SAFE_INTEGER,
                Number.MAX_SAFE_INTEGER,
                undefined,
                this,
                lineSegment
            );
        } else {
            const x = (b2 * c1 - b1 * c2) / determinant;
            const y = (a1 * c2 - a2 * c1) / determinant;
            return new IntersectionModel(x, y, undefined, this, lineSegment);
        }
    }


    insideSegment(intersecX, intersecY) {
        const valuesR = this.getDirection();
        const dirX = valuesR[0];
        const dirY = valuesR[1];
        const cInterA = dirX * intersecX + dirY * intersecY;

        // comparaca cv do ponto A > cv da intersec =>  vazio
        const cRA = dirX * this.center.posX + dirY * this.center.posY;

        if (cInterA < cRA) {
            return false;
        }

        // comparaca cv do ponto B < cv da intersec =>  vazio
        const cRB = dirX * this.radius.posX + dirY * this.radius.posY;

        if (cInterA > cRB) {
            this.posX = Number.MAX_SAFE_INTEGER;
            this.posY = Number.MAX_SAFE_INTEGER;
            return false;
        }

        return true;
    }

}